# TensorFlow Security Advisories

C++ fuzzing: [![Fuzzing Status](https://oss-fuzz-build-logs.storage.googleapis.com/badges/tensorflow.svg)](https://bugs.chromium.org/p/oss-fuzz/issues/list?sort=-opened&can=1&q=proj:tensorflow)

Python fuzzing: [![Fuzzing Status](https://oss-fuzz-build-logs.storage.googleapis.com/badges/tensorflow-py.svg)](https://bugs.chromium.org/p/oss-fuzz/issues/list?sort=-opened&can=1&q=proj:tensorflow-py)

We regularly publish security advisories about using TensorFlow.

*Note*: In conjunction with these security advisories, we strongly encourage
TensorFlow users to read and understand TensorFlow's security model as outlined
in [SECURITY.md](https://github.com/tensorflow/tensorflow/blob/master/SECURITY.md).

| Advisory Number | Type               | Versions affected | Reported by           | Additional Information      |
|-----------------|--------------------|:-----------------:|-----------------------|-----------------------------|
| [TFSA-2021-108](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-108.md)   | Segfault in `tf.raw_ops.ImmutableConst`                                            | <= 2.5.0            | (discovered internally)                       |  |
| [TFSA-2021-107](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-107.md)   | Segfault in `tf.raw_ops.SparseCountSparseOutput`                                   | <= 2.5.0            | (discovered internally)                       |  |
| [TFSA-2021-106](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-106.md)   | Crash in `tf.strings.substr` due to `CHECK`-fail                                   | <= 2.5.0            | (Reported on GitHub) | [issue report](https://github.com/tensorflow/tensorflow/issues/46900) |
| [TFSA-2021-105](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-105.md)   | Crash in `tf.transpose` with complex inputs                                        | <= 2.5.0            | (Reported on GitHub) | [issue report](https://github.com/tensorflow/tensorflow/issues/46891) |
| [TFSA-2021-104](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-104.md)   | Null dereference in Grappler's `TrySimplify`                                       | <= 2.5.0            | (discovered internally)                       |  |
| [TFSA-2021-103](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-103.md)   | Stack overflow in `ParseAttrValue` with nested tensors                             | <= 2.5.0            | (discovered internally)                       |  |
| [TFSA-2021-102](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-102.md)   | Interpreter crash from `tf.io.decode_raw`                                          | <= 2.5.0            | (discovered internally)                       |  |
| [TFSA-2021-101](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-101.md)   | Incomplete validation in `tf.raw_ops.CTCLoss`                                      | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-100](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-100.md)   | Heap buffer overflow in `BandedTriangularSolve`                                    | <= 2.5.0            | Ye Zhang and Yakun Zhang of Baidu X-Team      |  |
| [TFSA-2021-099](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-099.md)   | Invalid validation in `QuantizeAndDequantizeV2`                                    | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-098](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-098.md)   | Incomplete validation in `SparseReshape`                                           | >=2.3.0, <= 2.5.0   | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-097](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-097.md)   | Incomplete validation in `SparseSparseMinimum`                                     | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-096](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-096.md)   | Incomplete validation in `SparseAdd`                                               | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-095](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-095.md)   | Heap OOB and null pointer dereference in `RaggedTensorToTensor`                    | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-094](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-094.md)   | Heap OOB read in TFLite                                                            | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-093](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-093.md)   | Heap OOB write in TFLite                                                           | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-092](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-092.md)   | Integer overflow in TFLite memory allocation                                       | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-091](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-091.md)   | Integer overflow in TFLite concatentation                                          | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-090](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-090.md)   | Division by zero in TFLite's implementation of hashtable lookup                    | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-089](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-089.md)   | Division by zero in TFLite's implementation of `DepthwiseConv`                     | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-088](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-088.md)   | Division by zero in TFLite's implementation of `OneHot`                            | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-087](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-087.md)   | Division by zero in TFLite's implementation of `Split`                             | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-086](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-086.md)   | Division by zero in TFLite's implementation of `SVDF`                              | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-085](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-085.md)   | Division by zero in TFLite's implementation of `SpaceToBatchNd`                    | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-084](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-084.md)   | Division by zero in TFLite's implementation of `BatchToSpaceNd`                    | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-083](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-083.md)   | Division by zero in TFLite's implementation of `EmbeddingLookup`                   | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-082](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-082.md)   | Division by zero in TFLite's convolution code                                      | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-081](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-081.md)   | Division by zero in TFLite's implementation of `DepthToSpace`                      | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-080](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-080.md)   | Stack overflow due to looping TFLite subgraph                                      | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-079](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-079.md)   | Null pointer dereference in TFLite's `Reshape` operator                            | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-078](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-078.md)   | Heap OOB read in TFLite's implementation of `Minimum` or `Maximum`                 | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-077](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-077.md)   | Division by zero in TFLite's implementation of `TransposeConv`                     | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-076](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-076.md)   | Division by zero in TFLite's implementation of `GatherNd`                          | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-075](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-075.md)   | Division by zero in TFLite's implementation of `SpaceToDepth`                      | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-074](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-074.md)   | Division by zero in optimized pooling implementations in TFLite                    | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-073](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-073.md)   | Division by zero in padding computation in TFLite                                  | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-072](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-072.md)   | Heap buffer overflow and undefined behavior in `FusedBatchNorm`                    | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-071](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-071.md)   | `CHECK`-fail due to integer overflow                                               | <= 2.5.0            | University of Virginia and University of California, Santa Barbara |  |
| [TFSA-2021-070](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-070.md)   | Heap OOB read in `tf.raw_ops.Dequantize`                                           | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-069](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-069.md)   | Segfault in `CTCBeamSearchDecoder`                                                 | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-068](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-068.md)   | Heap buffer overflow in `MaxPoolGrad`                                              | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-067](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-067.md)   | Heap buffer overflow in `FractionalAvgPoolGrad`                                    | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-066](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-066.md)   | Undefined behavior and `CHECK`-fail in `FractionalMaxPoolGrad`                     | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-065](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-065.md)   | Heap buffer overflow in `AvgPool3DGrad`                                            | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-064](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-064.md)   | Heap buffer overflow in `MaxPool3DGradGrad`                                        | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-063](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-063.md)   | Undefined behavior in `MaxPool3DGradGrad`                                          | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-062](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-062.md)   | Division by 0 in `MaxPoolGradWithArgmax`                                           | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-061](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-061.md)   | Overflow/denial of service in `tf.raw_ops.ReverseSequence`                         | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-060](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-060.md)   | Reference binding to nullptr in `SdcaOptimizer`                                    | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-059](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-059.md)   | Memory corruption in `DrawBoundingBoxesV2`                                         | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-058](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-058.md)   | Heap out of bounds read in `RequantizationRange`                                   | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-057](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-057.md)   | Heap out of bounds read in `MaxPoolGradWithArgmax`                                 | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-056](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-056.md)   | Lack of validation in `SparseDenseCwiseMul`                                        | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-055](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-055.md)   | Reference binding to null in `ParameterizedTruncatedNormal`                        | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-054](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-054.md)   | Heap OOB access in `Dilation2DBackpropInput`                                       | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-053](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-053.md)   | Null pointer dereference in `SparseFillEmptyRows`                                  | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-052](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-052.md)   | Null pointer dereference in `EditDistance`                                         | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-051](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-051.md)   | `CHECK`-fail in `tf.raw_ops.RFFT`                                                  | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-050](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-050.md)   | `CHECK`-fail in `tf.raw_ops.IRFFT`                                                 | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-049](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-049.md)   | `CHECK`-fail in `LoadAndRemapMatrix`                                               | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-048](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-048.md)   | Heap buffer overflow in `RaggedTensorToTensor`                                     | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-047](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-047.md)   | Heap OOB access in unicode ops                                                     | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-046](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-046.md)   | Heap buffer overflow in `SparseSplit`                                              | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-045](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-045.md)   | Division by 0 in `Reverse`                                                         | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-044](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-044.md)   | Division by 0 in `SparseMatMul`                                                    | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-043](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-043.md)   | Division by 0 in `FusedBatchNorm`                                                  | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-042](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-042.md)   | Division by 0 in `DenseCountSparseOutput`                                          | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-041](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-041.md)   | `CHECK`-failure in `UnsortedSegmentJoin`                                           | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-040](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-040.md)   | Heap OOB in `QuantizeAndDequantizeV3`                                              | <= 2.5.0            | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-039](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-039.md)   | OOB read in `MatrixTriangularSolve`                                                | <= 2.5.0            | Ye Zhang and Yakun Zhang of Baidu X-Team      |  |
| [TFSA-2021-038](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-038.md)   | Division by 0 in `FractionalAvgPool`                                               | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-037](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-037.md)   | Division by 0 in `QuantizedAdd`                                                    | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-036](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-036.md)   | Division by 0 in `QuantizedBatchNormWithGlobalNormalization`                       | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-035](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-035.md)   | Heap out of bounds in `QuantizedBatchNormWithGlobalNormalization`                  | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-034](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-034.md)   | Division by 0 in `QuantizedBiasAdd`                                                | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-033](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-033.md)   | Heap buffer overflow in `SparseTensorToCSRSparseMatrix`                            | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-032](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-032.md)   | `CHECK`-fail in `CTCGreedyDecoder`                                                 | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-031](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-031.md)   | `CHECK`-fail in `QuantizeAndDequantizeV4Grad`                                      | >= 2.4.0, <= 2.5.0  | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-030](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-030.md)   | Null pointer dereference in `StringNGrams`                                         | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-029](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-029.md)   | Heap buffer overflow `StringNGrams`                                                | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-028](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-028.md)   | Heap buffer overflow `Conv2DBackpropFilter`                                        | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-027](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-027.md)   | Division by zero in `Conv2DBackpropFilter`                                         | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-026](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-026.md)   | Heap buffer overflow in `QuantizedReshape`                                         | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-025](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-025.md)   | Heap buffer overflow in `QuantizedResizeBilinear`                                  | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-024](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-024.md)   | `CHECK`-fail in `SparseConcat`                                                     | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-023](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-023.md)   | Heap buffer overflow in `QuantizedMul`                                             | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-022](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-022.md)   | `CHECK`-fail in `DrawBoundingBoxes`                                                | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-021](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-021.md)   | Heap out of bounds read in `RaggedCross`                                           | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-020](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-020.md)   | `CHECK`-fail in `tf.raw_ops.EncodePng`                                             | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-019](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-019.md)   | Heap buffer overflow caused by rounding                                            | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-018](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-018.md)   | Invalid validation in `SparseMatrixSparseCholesky`                                 | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-017](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-017.md)   | Division by 0 in `QuantizedMul`                                                    | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-016](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-016.md)   | Division by 0 in `QuantizedConv2D`                                                 | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-015](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-015.md)   | Division by 0 in `Conv2D`                                                          | <= 2.5.0            | Ying Wang and Yakun Zhang of Baidu X-Team     |  |
| [TFSA-2021-014](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-014.md)   | Division by 0 in `Conv2DBackpropInput`                                             | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-013](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-013.md)   | Division by 0 in `Conv2DBackpropFilter`                                            | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-012](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-012.md)   | `CHECK`-fail in `AddManySparseToTensorsMap`                                        | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-011](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-011.md)   | Division by 0 in `Conv3DBackprop*`                                                 | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-010](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-010.md)   | Heap buffer overflow in `Conv3DBackprop*`                                          | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-009](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-009.md)   | Segfault in `SparseCountSparseOutput`                                              | >= 2.3.0, <= 2.5.0  | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-008](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-008.md)   | `CHECK`-fail in `SparseCross` due to type confusion                                | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-007](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-007.md)   | Session operations in eager mode lead to null pointer dereferences                 | >= 2.0.0, <= 2.5.0  | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-006](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-006.md)   | Division by zero in `Conv3D`                                                       | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-005](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-005.md)   | Null pointer dereference via invalid Ragged Tensors                                | <= 2.5.0            | Yakun Zhang and Ying Wang of Baidu X-Team     |  |
| [TFSA-2021-004](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-004.md)   | Reference binding to null pointer in `MatrixDiag*` ops                             | <= 2.5.0            | Ye Zhang and Yakun Zhang of Baidu X-Team      |  |
| [TFSA-2021-003](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-003.md)   | Type confusion during tensor casts lead to dereferencing null pointers             | <= 2.5.0            | Aivul Team from Qihoo 360; Ye Zhang and Yakun Zhang of Baidu X-Team |  |
| [TFSA-2021-002](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-002.md)   | Heap out of bounds write in `RaggedBinCount`                                       | >= 2.3.0, <= 2.5.0  | Aivul Team from Qihoo 360                     |  |
| [TFSA-2021-001](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2021-001.md)   | Heap buffer overflow in `RaggedBinCount`                                           | >= 2.3.0, <= 2.5.0  | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-034](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-034.md)   | Heap out of bounds access in MakeEdge                                              | >= 1.15.0, <= 2.3.0 | (discovered internally)                       |  |
| [TFSA-2020-033](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-033.md)   | CHECK-fail in LSTM with zero-length input                                          | >= 1.15.0, <= 2.3.0 | (discovered internally)                       |  |
| [TFSA-2020-032](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-032.md)   | Heap out of bounds read in filesystem glob matching                                | 2.4.0-rc{0,1,2,3}   | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-031](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-031.md)   | Write to immutable memory region                                                   | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-030](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-030.md)   | Lack of validation in data format attributes                                       | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-029](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-029.md)   | Uninitialized memory access in Eigen types                                         | >= 1.15.0, <= 2.3.0 | (discovered internally)                       |  |
| [TFSA-2020-028](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-028.md)   | Float cast overflow undefined behavior                                             | <= 2.3 | (Reported on GitHub) | [issue report](https://github.com/tensorflow/tensorflow/issues/42129) |
| [TFSA-2020-027](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-027.md)   | Segfault in `tf.quantization.quantize_and_dequantize                              `| <= 2.3 | (Reported on GitHub) | [issue report](https://github.com/tensorflow/tensorflow/issues/42105) |
| [TFSA-2020-026](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-026.md)   | Segfault in `tf.raw_ops.Switch` in eager mode                                      | 2.2.0, 2.3.0        | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-025](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-025.md)   | Undefined behavior in `dlpack.to_dlpack`                                           | 2.2.0, 2.3.0        | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-024](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-024.md)   | Memory leak in `dlpack.to_dlpack`                                                  | 2.2.0, 2.3.0        | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-023](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-023.md)   | Memory corruption in `dlpack.to_dlpack`                                            | 2.2.0, 2.3.0        | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-022](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-022.md)   | Crash due to invalid shape of `grad_values` in SparseFillEmptyRowsGrad             | >= 1.15.0, <= 2.3.0 | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-021](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-021.md)   | Heap buffer overflow in SparseFillEmptyRowsGrad                                    | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-020](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-020.md)   | Heap buffer overflow in weighted sparse count ops                                  | 2.3.0               | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-019](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-019.md)   | Crash due to invalid splits in SparseCountSparseOutput                             | 2.3.0               | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-018](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-018.md)   | Heap buffer overflow due to invalid indices in SparseCountSparseOutput             | 2.3.0               | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-017](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-017.md)   | Abort due to invalid splits in RaggedCountSparseOutput                             | 2.3.0               | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-016](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-016.md)   | Segfault due to invalid splits in RaggedCountSparseOutput                          | 2.3.0               | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-015](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-015.md)   | Heap buffer overflow due to invalid splits in RaggedCountSparseOutput              | 2.3.0               | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-014](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-014.md)   | Integer truncation in Shard API usage                                              | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-013](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-013.md)   | Format-string vulnerability in TensorFlow's `as_string`                            | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-012](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-012.md)   | Segfault by calling session-only ops in eager mode                                 | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-011](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-011.md)   | Data leak in `tf.raw_ops.StringNGrams                                             `| >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-010](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-010.md)   | Incomplete validation in TensorFlow's SavedModel's constant nodes causes segfaults | >= 1.15.0, <= 2.3.0 | Shuaike Dong, Alipay Tian Qian Security Lab   | [issue report](https://github.com/tensorflow/tensorflow/issues/41097) |
| [TFSA-2020-009](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-009.md)   | Segfault and data corruption caused by negative indexing in TFLite                 | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-008](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-008.md)   | Data corruption due to dimension mismatch in TFLite                                | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-007](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-007.md)   | Null pointer dereference in TFLite                                                 | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360, variant analysis   |  |
| [TFSA-2020-006](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-006.md)   | Segmentation fault and/or data corruption due to invalid TFLite model              | >= 1.15.0, <= 2.3.0 | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-005](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-005.md)   | Out of bounds access in TFLite operators                                           | >= 1.15.0, <= 2.3.0 | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-004](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-004.md)   | Out of bounds access in TFLite implementation of segment sum                       | 2.2.0, 2.3.0        | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-003](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-003.md)   | Denial of service from TFLite implementation of segment sum                        | 2.2.0, 2.3.0        | (variant analysis, Aivul Team from Qihoo 360) |  |
| [TFSA-2020-002](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-002.md)   | Out of bounds write in TFLite implementation of segment sum                        | 2.2.0, 2.3.0        | Aivul Team from Qihoo 360                     |  |
| [TFSA-2020-001](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2020-001.md)   | Segmentation fault when converting a Python string to `tf.float16` | >= 1.12.0, <= 2.1 | (found internally) |  |
| [TFSA-2019-002](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2019-002.md)   | Heap buffer overflow in `UnsortedSegmentSum` | <= 1.14 | (found internally) |  |
| [TFSA-2019-001](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2019-001.md)   | Null Pointer Dereference Error in Decoding GIF Files | <= 1.12 | Baidu Security Lab |  |
| [TFSA-2018-006](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2018-006.md)   | Crafted Configuration File results in Invalid Memory Access | <= 1.7 | Blade Team of Tencent |  |
| [TFSA-2018-005](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2018-005.md)   | Old Snappy Library Usage Resulting in Memcpy Parameter Overlap | <= 1.7 | Blade Team of Tencent |  |
| [TFSA-2018-004](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2018-004.md)   | Checkpoint Meta File Out-of-Bounds Read | <= 1.7 | Blade Team of Tencent |  |
| [TFSA-2018-003](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2018-003.md)   | TensorFlow Lite TOCO FlatBuffer Parsing Vulnerability | <= 1.7 | Blade Team of Tencent |  |
| [TFSA-2018-002](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2018-002.md)   | GIF File Parsing Null Pointer Dereference Error | <= 1.5 | Blade Team of Tencent |  |
| [TFSA-2018-001](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/security/advisory/tfsa-2018-001.md)   | BMP File Parser Out-of-bounds Read | <= 1.6 | Blade Team of Tencent |  |
| -               | Out Of Bounds Read |             <= 1.4 | Blade Team of Tencent | [issue report](https://github.com/tensorflow/tensorflow/issues/14959) |

